import mkdocs.config.base
import mkdocs.config.config_options
import mkdocs.plugins
import mkdocs.structure.files
import mkdocs.structure.nav

from . import parser
from . import util


class LiterateNavPlugin(mkdocs.plugins.BasePlugin):
    config_scheme = (
        ("nav_file", mkdocs.config.config_options.File(exists=True)),
        ("exclude", mkdocs.config.config_options.Type(bool, default=False)),
    )

    def load_config(self, *args, **kwargs):
        errors, warnings = super().load_config(*args, **kwargs)
        if not self.config.get("nav_file"):
            error = mkdocs.config.base.ValidationError(
                "literate-nav plugin requires the nav_file parameter"
            )
            errors.append(("nav_file", error))
        return errors, warnings

    def on_files(self, files, config):
        result = []
        for f in files:
            if self.config["exclude"] and f.abs_src_path == self.config["nav_file"]:
                continue
            result.append(f)
        return mkdocs.structure.files.Files(result)

    def on_nav(self, nav, config, files):
        for page in nav.pages:
            page.previous_page = None
            page.next_page = None

        items = parser.markdown_to_nav(self.config["nav_file"])
        pages = []

        @util.collect
        def _convert_nav(items):
            for title, item in items:
                if isinstance(item, list):
                    section = mkdocs.structure.nav.Section(
                        title, children=_convert_nav(item)
                    )
                    for child in section.children:
                        child.parent = section
                    yield section
                else:
                    page = mkdocs.structure.nav.Page(
                        title, files.get_file_from_path(item), config
                    )
                    if pages:
                        page.previous_page = pages[-1]
                        pages[-1].next_page = page
                    pages.append(page)
                    yield page

        items = _convert_nav(items)
        return mkdocs.structure.nav.Navigation(items, pages)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mkdocs_literate_nav']

package_data = \
{'': ['*']}

install_requires = \
['mkdocs>=1.0,<2.0']

entry_points = \
{'mkdocs.plugins': ['literate-nav = '
                    'mkdocs_literate_nav.plugin:LiterateNavPlugin']}

setup_kwargs = {
    'name': 'mkdocs-literate-nav',
    'version': '0.1.0',
    'description': 'MkDocs plugin to specify the navigation in Markdown instead of YAML',
    'long_description': None,
    'author': 'Oleh Prypin',
    'author_email': 'oleh@pryp.in',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/oprypin/mkdocs-literate-nav',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



import io

import markdown.extensions
import markdown.treeprocessors

from . import util


def markdown_to_nav(input_file):
    ext = _MarkdownExtension()
    markdown.markdownFromFile(
        input=input_file, output=io.BytesIO(), extensions=[ext],
    )
    return ext.nav


class _MarkdownExtension(markdown.extensions.Extension):
    @property
    def nav(self):
        return self._treeprocessor.nav

    def extendMarkdown(self, md):
        self._treeprocessor = _Treeprocessor(md)
        md.treeprocessors.register(self._treeprocessor, "mkdocs_literate_nav", 0)


class _Treeprocessor(markdown.treeprocessors.Treeprocessor):
    LIST_TAGS = ("ul", "ol")

    @util.collect
    def _make_nav(self, section):
        for item in section:
            assert item.tag == "li"
            sub = _etree_children(item)
            if len(sub) == 3 and not sub[0] and sub[1].tag == "a" and not sub[2]:
                link = sub[1]
                sub = _etree_children(link)
                if len(sub) == 1:
                    yield (sub[0], link.get("href"))
                    continue
            if len(sub) == 3 and sub[0] and sub[1].tag in self.LIST_TAGS and not sub[2]:
                yield (sub[0], self._make_nav(sub[1]))
                continue

    def run(self, doc):
        self.nav = []
        for top_level_item in doc:
            if top_level_item.tag in self.LIST_TAGS:
                self.nav = self._make_nav(top_level_item)
        return doc


@util.collect
def _etree_children(el):
    text = el.text or ""
    yield text.strip() and text
    for child in el:
        yield child
        text = child.tail or ""
        yield text.strip() and text
